/* Shape manipulation & display */
#include <stdio.h>
#include "blocks.h"

/* Add or remove shape rotated by q to board to (spotX, spotY) */
void doshape(int spotX, int spotY, SHAPE *sh, int q, int setf)
{
    int i, x, y;
    int ch = setf==ADD ? '0' + sh->id : EMPTY;

    /* Adjust offset for corner */
    spotX -= sh->cnr[q][0];
    spotY -= sh->cnr[q][1];

    for (i=0; i<sh->p; i++)
    {
        rotate(sh->cell[i], sh, q, &x, &y);
        *pbrd(spotX+x, spotY+y) = ch;
    }
}

/* Test shape number s to see if it fits at (spotX, spotY).
   Returns OK on success */
int testshape(int spotX, int spotY, SHAPE *sh, int q)
{
    int i, x, y;

    /* Adjust offset for corner */
    spotX -= sh->cnr[q][0];
    spotY -= sh->cnr[q][1];

    for (i=0; i<sh->p; i++)
    {
        rotate(sh->cell[i], sh, q, &x, &y);
        if (*pbrd(spotX+x, spotY+y) != EMPTY)
            return !OK;
    }

    return OK;
}

/* Text display of SHAPE data */
void dumpshape(SHAPE *sh)
{
    int i;

    if (!sh)
        return;

    printf("Shape %d\nSize=%d Width=%d Height=%d Symmetry=%d\n",
            sh->id, sh->p, sh->w+1, sh->h+1, sh->q);

    printf("Cells:\n");
    for (i=0; i<sh->p; i++)
        printf("%d: (%d,%d)\n", i, sh->cell[i][0], sh->cell[i][1]);

    printf("Corners:\n");
    for (i=0; i<sh->q; i++)
        printf("%d: (%d,%d)\n", i, sh->cnr[i][0], sh->cnr[i][1]);

    putchar('\n');
}

/* Display shapes in graphic form */
void showshapes(SHAPE *sh)
{
    int i, q;
    int spotX=0, spotY=0;

    printf("\nAll shapes, in all rotations\n");
    for (i=0; i<smax; i++, sh++)
    {
        printf("\nShape %d\nSize=%d Width=%d Height=%d Symmetry=%d\n",
                sh->id, sh->p, sh->w+1, sh->h+1, sh->q);
        
        for (q=0; q<sh->q; q++)
        {
            printf("Rotation %d\n", q);
            if (testshape(spotX, spotY, sh, q) == OK)
            {
                doshape(spotX, spotY, sh, q, ADD);
                putboard();
                doshape(spotX, spotY, sh, q, REM);
            }
            else
                printf("Doesn't fit\n\n");
        }
    }
    putchar('\n');
}

